import { EditorView, basicSetup } from 'codemirror';
import { javascript } from '@codemirror/lang-javascript';
import { json } from '@codemirror/lang-json';
import { html } from '@codemirror/lang-html';
import { markdown } from '@codemirror/lang-markdown';
import { css } from '@codemirror/lang-css';
let editor: EditorView = null;

type CreateOpts = {
  jsx?: boolean;
  typescript?: boolean;
  type?: 'javascript' | 'json' | 'html' | 'markdown' | 'css';
};
/**
 * 创建单例
 * @param el
 * @returns
 */
const createEditorInstance = (el?: HTMLDivElement, opts?: CreateOpts) => {
  if (editor && el) {
    el.appendChild(editor.dom);
    return editor;
  } else if (editor) {
    return editor;
  }
  const { type = 'javascript' } = opts || {};
  const plugins = [basicSetup];

  switch (type) {
    case 'json':
      plugins.push(json());
      break;
    case 'javascript':
      plugins.push(javascript({ jsx: opts?.jsx, typescript: opts?.typescript }));
      break;
    case 'css':
      plugins.push(css());
      break;
    case 'html':
      plugins.push(html());
      break;
    case 'markdown':
      plugins.push(markdown());
      break;
  }
  editor = new EditorView({
    extensions: plugins,
    parent: el || document.body,
  });
  editor.dom.style.height = '100%';
  return editor;
};

/**
 * 每次都创建新的实例
 * @param el
 * @returns
 */
export const createEditor = (el: HTMLDivElement, opts?: CreateOpts) => {
  const { type = 'javascript' } = opts || {};
  const plugins = [basicSetup];
  switch (type) {
    case 'json':
      plugins.push(json());
      break;
    case 'javascript':
      plugins.push(javascript({ jsx: opts?.jsx, typescript: opts?.typescript }));
      break;
    case 'css':
      plugins.push(css());
      break;
    case 'html':
      plugins.push(html());
      break;
    case 'markdown':
      plugins.push(markdown());
      break;
  }
  const editor = new EditorView({
    extensions: plugins,
    parent: el || document.body,
  });
  editor.dom.style.height = '100%';
  return editor;
};

export { editor, createEditorInstance };
